////////////////////////////////////////////////////////////////////////
// heure.sce                                         _____
//                                                  /      \
//  cree le       27/01/2013 par O. THUAL   O======| 14:30 |======[
//                                                  \______/
//  Tt=30: valeur vraie en minutes
//  T1o : premiere mesure de l'heure
//  T2o : deuxieme mesure de l'heure
//
//  sig1 : erreur de mesure sur T1
//  sig2 : erreur de mesure sur T2
//
// fonction cout 
// J(T)= 1/2 (T-T1)^2 / sigT1^2 + 1/2 (T-T2)^2 / sigT2^2 
//
// Analyse :  Ta = ( C1 T1 + C2 T2) / ( C1 + C2 )
//   avec C1 = 1 / sig1^2   
//    et  C2 = 1 / sig2^2 
//
// Visualisation des resultats : 
//    Etat vrai : bleu,   Analyse : rouge
//    Mesure T1 : vert,   Mesure T2 : jaune
////////////////////////////////////////////////////////////////

////////////////////////////
// Preparatifs pour scilab 
///////////////////////////

// Fermer les fenetres graphiques
xdel(winsid())

// Supprimer les variables precedentes
clear a

// Afficher le titre sur la console
disp("Assimilation de donnees : Quelle heure est-il ? ")

 /////////////////////////////////////////////////////////////
// saisie clavier : inter=%T,  sans saisie clavier : inter=%F 
/////////////////////////////////////////////////////////////
inter=%F;  

/////////////////////////////////////////////////
// traces X1 et X2 si histoi=%T,
// traces Xa          histoa=%T,
// passe si                 =%F
//////////////////////////////////////////////////
histoi=%T;
histoa=%T;

// Initialisation des numeros de figures
nfig=1;

//////////////////////////////
// Lancement de l'experience 
/////////////////////////////

// Nombre de tirages
jtirmax=10;

// Lancement du generateur de tirage aleatoire
rand('normal')
rand('seed',getdate('s'))


// Heure vraie
Tt=30;
disp(sprintf('Heure vraie Tt=%3.1f',Tt))


///////////////////////////
// valeurs par defaut
sig1_d=5;    // sig1_d=5  
sig2_d=7;    // sig2_d=7
/////////////////////////

///////////////////////////////
if(inter) // si inter=%F    //
// Mode interactif         //
///////////////////////////

// Saisie de l'heure T2    //
////////////////////////////

// saisie clavier de sig1
disp(sprintf("Erreur de la montre 1 : sig1 ? (Touche Enter pour valeur par defaut)"))
sig1_text=sprintf('Par defaut sig1=%2.1f,  sig1= ',sig1_d);
sig1=input(sig1_text);

// saisie clavier de sig2
disp(sprintf("Erreur de la montre 2 : sig2 ? (Touche Enter pour valeur par defaut)"))
sig2_text=sprintf('Par defaut sig2=%2.1f,  sig2= ',sig2_d);
sig2=input(sig2_text);

// affectation des valeurs par defaut si vide
if(isempty(sig1))
   sig1=sig1_d;
end
if(isempty(sig2))
   sig2=sig2_d;
end


///////////////////////////////
else      // si inter=%F    //
// Mode parametres fixes   //
///////////////////////////

// valeurs par defaut
sig1=sig1_d; 
sig2=sig2_d; 

end

// affichage des valeurs des erreurs 
disp(sprintf("Erreur de la montre 1 : sig1=%2.1f",sig1))
disp(sprintf("Erreur de la montre 2 : sig2=%2.1f",sig2))


// initialiation du cumul des erreurs pour le calcul du score
cumul=0;


 /////////////////////////////
// Boucle sur les tirages //
for jtir=1:jtirmax
/////////////////////////////


// visualisation de T1 et T2 seulement pour un tirage
if(jtir >= 2)
 histoi=%F;
end



// Heures T1 et T2
T1=Tt+sig1*rand(1,1);
T2=Tt+sig2*rand(1,1);

//affichage des valeurs   
disp(sprintf("T1=%2.1f, T2=%2.1f",T1,T2))

 ///////////////
// Analyse   //
//////////////

// certitudes
C1=1/sig1^2;
C2=1/sig2^2;
Ca=C1+C2;

// analyse
Ta=(C1*T1+C2*T2)/(C1+C2);
siga=1/sqrt(Ca);
disp(sprintf('    Ta=%3.1f, siga=%3.1f',Ta,siga))


 /////////////////////////////
// Visualisations         //
////////////////////////////

// nombre de tirages pour les visualisations
N=20;

// axe des X
Ymin=-.01; Ymax=1/sqrt(2*%pi)/siga;
DelX=5*siga; 
Xmin=Tt-DelX; Xmax=Tt+DelX;
NX=101;
X=linspace(Xmin,Xmax,NX);


 /////////////////////////
// visualisation de T1 //
////////////////////////
    

// variable aleatoire X1
R1=rand(1,N);
X1=Tt+sig1*R1;

// densite de probalite fX1
fX1max=1/sqrt(2*%pi)/sig1;
fX1=fX1max*exp(-.5*(X-Tt).^2/sig1^2);

////////////////////////////////////
if(histoi) // traces si histoi=%T  //
//////////////////////////////////

// preparation du graphique
figure(nfig,'BackgroundColor',[1 1 1]); 
nfig=nfig+1;
set(gca(),'grid',[1 1]*color('gray'))
set(gca(),"auto_clear","off")

// variable aleatoire X1
plot(X1,0*X1,'ok')
plot(X,fX1,'-g','LineWidth',3)

// trace de deux ecart-types sig1
plot([Tt-sig1 Tt+sig1],[fX1max/2 fX1max/2],'-dg','LineWidth',3)

// trace du tirage T1
plot([Tt Tt],[0 fX1max/4],'-db','LineWidth',3)
plot([T1 T1],[0 fX1max/4],'-dg','LineWidth',3)

// reglage axes de la figure
a=get("current_axes"); //get the handle of the newly created axes
a.data_bounds=[Xmin Xmax Ymin Ymax];

// Sauvegarde la figure en pdf si ficfigure=%T
ficfigure=%F; 
if ficfigure 
  xs2pdf(gcf(),'quelleurT1');
end

////////////////////////////////////
end      // rien si histoi=%F     //
//////////////////////////////////

 /////////////////////////
// visualisation de T2 //
////////////////////////
    
// tirage aleatoires X2
R2=rand(1,N);
X2=Tt+sig2*R2;

// densite de probalite fX2
fX2max=1/sqrt(2*%pi)/sig2;
fX2=fX2max*exp(-.5*(X-Tt).^2/sig2^2);

////////////////////////////////////
if(histoi) // traces si histoi=%T  //
//////////////////////////////////

// preparation du graphique
figure(nfig,'BackgroundColor',[1 1 1]);
nfig=nfig+1;
set(gca(),'grid',[1 1]*color('gray'))
set(gca(),"auto_clear","off")

// variable aleatoire X2
plot(X2,0*X2,'ok')
plot(X,fX2,'-y','LineWidth',3)

// trace de deux ecart-types sig2
plot([Tt-sig2 Tt+sig2],[fX2max/2 fX2max/2],'-dy','LineWidth',3)

// trace du tirage
plot([Tt Tt],[0 fX2max/4],'-db','LineWidth',3)
plot([T2 T2],[0 fX2max/4],'-dy','LineWidth',3)

// reglage axes de la figure
a=get("current_axes"); //get the handle of the newly created axes
a.data_bounds=[Xmin Xmax Ymin Ymax];

// Sauvegarde la figure en pdf si ficfigure=%T
ficfigure=%F 
if ficfigure 
  xs2pdf(gcf(),'quelleurT2');
end

////////////////////////////////////
end      // rien si histoi=%F     //
//////////////////////////////////

 /////////////////////////
// visualisation de Ta //
////////////////////////
    
// preparation du graphique
figure(nfig,'BackgroundColor',[1 1 1]);
nfig=nfig+1;
set(gca(),'grid',[1 1]*color('gray'))
set(gca(),"auto_clear","off")

// variable aleatoire Xa
Ra=rand(1,N);
Xa=Tt+siga*Ra;

// densite de probalite
fXamax=1/sqrt(2*%pi)/siga;
fXa=fXamax*exp(-.5*(X-Tt).^2/siga^2);

// trace des lois de probabilite
plot(Xa,0*Xa,'ok')
plot(X,fXa,'-r','LineWidth',3)
plot(X,fX1,'-g','LineWidth',3)
plot(X,fX2,'-y','LineWidth',3)

// trace de deux ecart-types
plot([Tt-siga Tt+siga],[fXamax/2 fXamax/2],'-dr','LineWidth',3)

// trace du tirage
plot([Tt Tt],[0 fXamax/4],'-db','LineWidth',3)
plot([Ta Ta],[0 fXamax/4],'-dr','LineWidth',3)
plot([T1 T1],[0 fXamax/4],'-dg','LineWidth',3)
plot([T2 T2],[0 fXamax/4],'-dy','LineWidth',3)

// reglage axes de la figure
DelaX=3*siga; 
Xamin=Tt-DelaX; Xamax=Tt+DelaX;
a=get("current_axes"); //get the handle of the newly created axes
a.data_bounds=[Xamin Xamax Ymin Ymax];

// Sauvegarde la figure en pdf si ficfigure=%T
ficfigure=%F 
if ficfigure 
  xs2pdf(gcf(),'quelleurTa');
end

// cumul des erreurs pour le calcul du score
cumul=cumul+(Ta-Tt)^2;

/////////////////////
end
// Fin de la boucle //
/////////////////////

// Impression du score moyen
score=sqrt(cumul/jtirmax);
disp(sprintf('score moyen =%5.3g',score))


return
